<?php

/**
 * @package    WPBuddy Plugin
 * @subpackage Google+ Author
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
} // Exit if accessed directly


/**
 * @version 2.4.9
 */
class WPB_Google_Author extends WPB_Plugin {

	/**
	 * _plugin_version
	 * The plugin version
	 *
	 * (default value: '1.0')
	 *
	 * @var string
	 * @since 1.2
	 */
	public $_plugin_version = '2.4.9';


	/**
	 * _plugin_name
	 *
	 * (default value: google-author)
	 *
	 * @var string
	 * @since 1.2
	 */
	protected $_plugin_name = 'google-author';


	/**
	 * _plugin_textdomain
	 *
	 * (default value: google-author )
	 *
	 * @var string
	 * @since 2.0
	 */
	protected $_plugin_textdomain = 'google-author';


	/**
	 * __construct function.
	 * Just do the normal startup stuff (adding actions and so on ...)
	 *
	 * @access public
	 *
	 * @param null   $file
	 * @param null   $plugin_url
	 * @param string $inclusion
	 *
	 * @param bool   $auto_update
	 *
	 * @return \WPB_Google_Author
	 * @since  1.0
	 */
	public function __construct( $file = null, $plugin_url = null, $inclusion = 'plugin', $auto_update = true ) {

		// call the parent constructor first
		parent::__construct( $file, $plugin_url, $inclusion, $auto_update );

		$this->_purchase_code_settings_page_url = admin_url( 'options-general.php?page=gpaisrpro' );

		// add widgets
		$this->add_widgets();

		// do the admin stuff
		$this->do_admin();

		// do the non-admin stuff
		$this->do_non_admin();

	}


	/**
	 * Does some admin stuff
	 * @since 1.2
	 * @return void
	 */
	private function do_admin() {
		if ( ! is_admin() ) {
			return;
		}

		add_filter( 'admin_init', array( &$this, 'admin_init' ) );

		add_filter( 'user_contactmethods', array( &$this, 'user_contactmethods' ) );

		// Create the admin menu in backend
		add_action( 'admin_menu', array( &$this, 'menu' ) );

		// Add Ajax-Support for the theme options page
		add_action( 'wp_ajax_gpaisrpro_options', array( &$this, 'ajax_post' ) );

		// add settings links to the plugins admin page
		if ( 'plugin' == $this->_inclusion ) {
			add_filter( 'plugin_action_links_' . plugin_basename( $this->_plugin_file ), array( &$this, 'plugin_action_links' ) );
		}

		// Enqueue the backend JS and CSS Scripts
		add_action( 'admin_enqueue_scripts', array( &$this, 'enqueue_scripts_backend' ), 100 );

		// Add a pointer message
		add_action( 'admin_footer', array( &$this, 'wp_pointer_message' ) );

		add_action( 'wp_ajax_wpbga_ajax_postlist', array( &$this, 'ajax_postlist' ) );
		add_action( 'wp_ajax_wpbga_ajax_userlist', array( &$this, 'ajax_userlist' ) );
	}


	/**
	 * Does some non-admin stuff
	 * @since 1.2
	 * @return void
	 *
	 */
	private function do_non_admin() {
		if ( is_admin() ) {
			return;
		}

		// adds author headers to tag, category and other special pages
		add_action( 'wp_head', array( &$this, 'author_header_special' ) );

		$options = get_option( 'gpaisrpro' );

		if ( ! isset( $options['google_plus_page_header'] ) OR 1 != $options['google_plus_page_header'] ) {
			add_action( 'wp_footer', array( &$this, 'add_google_publishership' ) );
		}
		else {
			add_action( 'wp_head', array( &$this, 'publisher_header' ) );
		}

		// replacement == 1 : replace the author link in the theme
		if ( isset( $options['replacement'] ) && 1 == $options['replacement'] ) {
			add_filter( 'author_link', array( &$this, 'author_link_filter' ), 10, 3 );
			return;
		}

		// replacement == 3 : author link to the header
		if ( isset( $options['replacement'] ) && 3 == $options['replacement'] ) {
			add_action( 'wp_head', array( &$this, 'author_header' ) );
			return;
		}

		// replacement == 0 : author link to the content
		add_filter( 'the_content', array( &$this, 'author_link_in_content' ), 15 );

	}


	/**
	 * Adds the settings fields
	 *
	 * @since 2.4.8
	 */
	public function admin_init() {
		$settings = new WPB_Google_Author_Settings( $this );
		$settings->settings();
	}


	/**
	 * scripts function.
	 * adds the scripts which are important for add_metabox (in backend).
	 * this function will be called from menu function (see below) after the pagehook was set to only import them on the specific page.
	 *
	 * @access public
	 * @return void
	 * @since  1.0
	 */
	public function scripts_settings() {

		wp_enqueue_script( 'common' );

		wp_enqueue_script( 'wp-lists' );

		wp_enqueue_script( 'postbox' );

		// Creates the settings (and the settings page)
		$settings = new WPB_Google_Author_Settings( $this );
		$settings->settings();

	}


	/**
	 * menu function.
	 * will add the menu in the WordPress backend
	 *
	 * @access public
	 *
	 * @param mixed $id
	 *
	 * @return void
	 * @since  1.0
	 */
	public function menu( $id ) {

		// create the menu (which generates the pagehook)
		$this->_pagehook = add_options_page( __( 'Google+ Author Information', $this->get_textdomain() ), __( 'Google+ Author Information', $this->get_textdomain() ), 'administrator', 'gpaisrpro', array( $this, 'settings_page' ) );

		// only add the scripts for the google author plugins page in backend
		add_action( 'load-' . $this->_pagehook, array( &$this, 'scripts_settings' ) );
	}


	/**
	 * settings_page function.
	 * creates the settings page content and some metaboxes
	 *
	 * @access public
	 * @return void
	 * @since  1.0
	 */
	public function settings_page() {

		$metabox_class = new WPB_Google_Author_Metaboxes( $this );

		// add meteabox "General"
		add_meta_box( 'gpaisrpro_general', __( 'General' ), array( $metabox_class, 'general' ), $this->_pagehook, 'normal', 'core' );

		// add metabox "howto"
		add_meta_box( 'gpaisrpro_howto', __( 'Help', $this->get_textdomain() ), array( $metabox_class, 'howto' ), $this->_pagehook, 'normal', 'low' );

		// add metabox "Test"
		add_meta_box( 'gpaisrpro_test', __( 'Test Authorship', $this->get_textdomain() ), array( $metabox_class, 'test' ), $this->_pagehook, 'normal', 'low' );

		// Add metabox "Authorship settings"
		add_meta_box( 'gpaisrpro_settings', __( 'Authorship Settings', $this->get_textdomain() ), array( $metabox_class, 'settings' ), $this->_pagehook, 'normal', 'low' );

		// Add metabox "Publisher settings"
		add_meta_box( 'gpaisrpro_settings_publisher', __( 'Publisher Settings', $this->get_textdomain() ), array( $metabox_class, 'settings_publisher' ), $this->_pagehook, 'normal', 'low' );

		// Add metabox "General settings"
		add_meta_box( 'gpaisrpro_settings_general', __( 'General Settings', $this->get_textdomain() ), array( $metabox_class, 'settings_general' ), $this->_pagehook, 'normal', 'low' );

		// Add metabox "about"
		add_meta_box( 'gpaisrpro_about', __( 'About', $this->get_textdomain() ), array( $metabox_class, 'about' ), $this->_pagehook, 'side', 'default' );

		// Add metabox "social"
		add_meta_box( 'gpaisrpro_social', __( 'Like this plugin?', $this->get_textdomain() ), array( $metabox_class, 'social' ), $this->_pagehook, 'side', 'default' );

		// Add metabox "links"
		add_meta_box( 'gpaisrpro_links', __( 'Helpful links', $this->get_textdomain() ), array( $metabox_class, 'links' ), $this->_pagehook, 'side', 'default' );

		// Add metabox "subscribe"
		add_meta_box( 'gpaisrpro_subscribe', __( 'Keep up-to-date', $this->get_textdomain() ), array( $metabox_class, 'subscribe' ), $this->_pagehook, 'side', 'default' );
		?>

		<div class="wrap" id="gpaisrpro">
			<?php screen_icon( 'options-general' ); ?>
			<form action="options.php" method="post" class="gpaisrpro-form">
				<h2><?php echo __( 'Google+ Author Information Settings', $this->get_textdomain() ); ?></h2><br />

				<div id="poststuff" class="metabox-holder has-right-sidebar">

					<div id="side-info-column" class="inner-sidebar">
						<?php do_meta_boxes( $this->_pagehook, 'side', array() ); ?>
					</div>

					<div id="post-body" class="has-sidebar">
						<div id="post-body-content" class="has-sidebar-content">
							<?php do_meta_boxes( $this->_pagehook, 'normal', array() ); ?>
						</div>
					</div>

					<br class="clear" />

					<?php
					settings_fields( 'gpaisrpro_options_group' );
					wp_nonce_field( 'closedpostboxes', 'closedpostboxesnonce', false );
					wp_nonce_field( 'meta-box-order', 'meta-box-order-nonce', false );
					?>
					<input type="hidden" value="1" name="gpaisrpro_save" />
					<input type="hidden" value="<?php echo wp_create_nonce( 'gpaisrpro-options-ajax-nonce' ); ?>" name="gpaisrpro-options-ajax-nonce" id="gpaisrpro-options-ajax-nonce" />
				</div>
				<!-- /poststuff -->
			</form>
		</div><!-- /wrap -->

	<?php
	}


	/**
	 * Just replaces the link with the google+ url
	 *
	 * @param $link
	 * @param $author_id
	 * @param $author_nicename
	 *
	 * @return string
	 * @since 1.0
	 */
	function author_link_filter( $link, $author_id, $author_nicename ) {

		// check this is the right post type
		// get the options
		$options = get_option( 'gpaisrpro' );

		// if nothing was set: posts and pages are standard
		if ( ! isset( $options['post_types'] ) ) {
			$options['post_types'] = array( 'post', 'page' );
		}

		global $post;

		// if the current post type does not match the selected ones, just return the link
		if ( ! in_array( $post->post_type, $options['post_types'] ) ) {
			return $link;
		}

		// get the author link
		$gplus_author_link = get_the_author_meta( 'googleplus', $author_id );

		if ( ! empty( $gplus_author_link ) ) {
			return $gplus_author_link . '?rel=author';
		}

		// if the author links was not set, just return the link
		return $link;
	}

	/**
	 * Writes the Google+-URL right after the Content
	 *
	 * @param          $content
	 *
	 * @global WP_Post $post
	 *
	 * @return string
	 * @since 1.0
	 */
	public static function author_link_in_content( $content ) {

		// load the options from the settings page
		$options = get_option( 'gpaisrpro' );

		global $post;

		if ( ! $post instanceof WP_Post ) {
			return $content;
		}

		// if nothing was set: posts and pages are standard
		if ( ! isset( $options['post_types'] ) ) {
			$options['post_types'] = array( 'post', 'page' );
		}

		// if the current post type does not match the selected ones, just return the content
		if ( ! in_array( $post->post_type, $options['post_types'] ) ) {
			return $content;
		}

		// is this the feed? if yes, should we display the link?
		if ( ! isset( $options['in_feed'] ) ) {
			$options['in_feed'] = 0;
		}
		if ( is_feed() && $options['in_feed'] != 1 ) {
			return $content;
		}

		// find the link
		$gplus_author_link = get_the_author_meta( 'googleplus', $post->post_author );
		if ( empty( $gplus_author_link ) ) {
			return $content;
		}
		else {
			$gplus_author_link = $gplus_author_link . '?rel=author';
		}

		// open in a new window?
		$newWindow = '';
		if ( isset( $options['new_window'] ) && $options['new_window'] == 1 ) {
			$newWindow = 'target="_blank"';
		}

		// show or hide?
		$showHide = '';
		if ( isset( $options['hide'] ) && $options['hide'] == 1 ) {
			$showHide = 'style="display:none;"';
		}

		// check if this is the widget-link
		if ( isset( $options['replacement'] ) && $options['replacement'] == 2 ) {
			// if yes, always show the link
			$showHide = '';

			// and only show one link
			$options['before_after'] = 'before';
		}

		// is there a linktext?
		$linktext = 'Google+';
		if ( ! empty( $options['link_text'] ) ) {
			$linktext = $options['link_text'];
		}

		// where to display the link
		if ( ! isset( $options['before_after'] ) ) {
			$options['before_after'] = 'after';
		}

		// build the link
		$link = '<a rel="author" class="gpaisrpro gpaisrpro-frontend-link" href="' . $gplus_author_link . '" '
				. $newWindow . ' '
				. $showHide . '>' . $linktext . '</a>';

		return
				( ( $options['before_after'] == 'all' OR $options['before_after'] == 'before' ) ? $link : '' )
				. $content
				. ( ( $options['before_after'] == 'all' OR $options['before_after'] == 'after' ) ? $link : '' );

	}

	/**
	 * ajax_post function.
	 * This will update all theme options from the theme options page and will test the current page
	 *
	 * @access public
	 * @return void
	 * @since  1.0
	 */
	public function ajax_post() {

		// check if there is a legal request
		check_ajax_referer( 'gpaisrpro-options-ajax-nonce', 'security' );

		// split the form_data post element into the items
		parse_str( $_REQUEST['form_data'], $form_data );

		// save options first
		if ( isset( $form_data['gpaisrpro'] ) ) {
			update_option( 'gpaisrpro', $form_data['gpaisrpro'] );
		}

		if ( ! isset( $_REQUEST['post_id'] ) ) {
			wp_send_json_error();
		}

		$post_id = intval( $_REQUEST['post_id'] );
		$post    = get_post( $post_id );

		$messages = array();

		if ( $post_id <= 0 OR is_null( $post ) ) {
			$messages[] = array( 'success' => false, 'message' => __( 'Could not test this page.', $this->get_textdomain() ) );
			wp_send_json_error( array( 'messages' => $messages ) );
		}

		// check if the form data was set correctly
		if ( ! isset( $form_data['gpaisrpro']['post_types'] ) OR ! is_array( $form_data['gpaisrpro']['post_types'] ) ) {
			$messages[] = array( 'success' => false, 'message' => __( 'Please choose at least one post type below.', $this->get_textdomain() ) );
			wp_send_json_error( array( 'messages' => $messages ) );
		}

		global $wp_version;

		$options = array(
			'timeout'    => 30,
			'user-agent' => 'WordPress/' . $wp_version . '; ' . home_url()
		);

		$request = wp_remote_get( get_permalink( $post_id ), $options );

		// this is the page body. (the actual content). If there was an error during the request, this string will be empty
		$page = wp_remote_retrieve_body( $request );

		// check if an error occurred
		if ( is_wp_error( $request ) ) {
			$errors = $request->get_error_messages();
			// run through the errors array to get the messages
			if ( is_array( $errors ) ) {
				foreach ( $errors as $error ) {
					$messages[] = array( 'success' => false, 'message' => $error );
				}
			};

			wp_send_json_error( array( 'messages' => $messages ) );

		}

		if ( wp_remote_retrieve_response_code( $request ) != 200 ) {
			$messages[] = array( 'success' => false, 'message' => sprintf( __( 'The page with the URL %1$s could not be verified. The site delivered a page with the status code %2$s (%3$s).', $this->get_textdomain() ), '<a href="' . get_permalink( $post_id ) . '" target="_blank">' . get_permalink( $post_id ) . '</a>', wp_remote_retrieve_response_code( $request ), wp_remote_retrieve_response_message( $request ) ) );
			wp_send_json_error( array( 'messages' => $messages ) );
		}

		if ( empty( $page ) ) {
			$messages[] = array( 'success' => false, 'message' => __( 'The page you have selected seems to be empty.', $this->get_textdomain() ) );
			wp_send_json_error( array( 'messages' => $messages ) );
		}

		$link = get_the_author_meta( 'googleplus', $post->post_author );

		if ( empty( $link ) ) {
			$messages[] = array( 'success' => false, 'message' => sprintf( __( 'The author of the following post has not yet set the link to his/her Google+ profile: %s', $this->get_textdomain() ), '<a target="_blank" href="' . get_permalink( $post_id ) . '">' . get_permalink( $post_id ) . '</a>' ) );
			wp_send_json_error( array( 'messages' => $messages ) );
		}

		// search for this term
		$found = stripos( $page, $link );

		if ( false === $found ) {
			$messages[] = array( 'success' => false, 'message' => sprintf( __( 'The Google+ link could not be found on this page: %1$s. Please make sure that the author of this post has entered his/her Google+ link.', $this->get_textdomain() ), '<a target="_blank" href="' . get_permalink( $post_id ) . '">' . get_permalink( $post_id ) . '</a>' ) );
			wp_send_json_error( array( 'messages' => $messages ) );
		}

		$messages[] = array( 'success' => true, 'message' => __( 'The authors Google+ link was found. Perfect!', $this->get_textdomain() ) );
		$messages[] = array( 'success' => true, 'message' => __( 'Now you can check this page with the Google Rich Snippets Testing Tool.', $this->get_textdomain() ) . ' <a href="http://www.google.de/webmasters/tools/richsnippets?q=' . urlencode( get_permalink( $post_id ) ) . '" class="button" target="_blank">' . __( 'Test now', $this->get_textdomain() ) . '</a>' );

		// check for Google publishership
		if ( isset( $form_data['gpaisrpro']['google_plus_page_url'] ) && ! empty( $form_data['gpaisrpro']['google_plus_page_url'] ) ) {
			$found = stripos( $page, $form_data['gpaisrpro']['google_plus_page_url'] );
			if ( false === $found ) {
				$messages[] = array( 'success' => false, 'message' => __( 'The link to your Google+ page (Publishership) was not found.', $this->get_textdomain() ) );
			}
			else {
				$messages[] = array( 'success' => true, 'message' => __( 'The link to your Google+ page (Publishership) was found. Nice!', $this->get_textdomain() ) );
			}
		}

		wp_send_json_success( array( 'messages' => $messages ) );

	}

	private function add_widgets() {
		// read the options
		$options = get_option( 'gpaisrpro' );

		// Set 0 to standard
		if ( ! isset( $options['replacement'] ) ) {
			$options['replacement'] = 0;
		}

		// Check if Widget was selected
		if ( 2 == $options['replacement'] ) {
			add_action( 'widgets_init', array( &$this, 'register_google_plus_widget' ) );
		}
	}

	/**
	 * Registers the Google+ Author Widget
	 *
	 * @access public
	 * @return void
	 * @since  1.0
	 */
	public function register_google_plus_widget() {
		if ( ! class_exists( 'WPB_Google_Author_Widget' ) ) {
			return;
		}
		register_widget( 'WPB_Google_Author_Widget' );
	}


	/**
	 * Adds links to the plugins menu (where the plugins are listed)
	 *
	 * @param array $links
	 *
	 * @since 1.2
	 * @return array
	 */
	public function plugin_action_links( $links ) {
		$links[] = '<a href="' . get_admin_url( null, 'options-general.php?page=' . str_replace( 'settings_page_', '', $this->_pagehook ) ) . '">' . __( 'Settings', $this->get_textdomain() ) . '</a>';
		$links[] = '<a href="http://wp-buddy.com/products/" target="_blank">' . __( 'More Plugins by WPBuddy', $this->get_textdomain() ) . '</a>';
		return $links;
	}


	/**
	 * Enqueues the Javascripts and CSS files for the backend
	 *
	 * @param string $hook_suffix
	 *
	 * @since 1.2
	 * @return void
	 */
	public function enqueue_scripts_backend( $hook_suffix ) {
		if ( 'settings_page_gpaisrpro' != $hook_suffix ) {
			return;
		}

		wp_register_script( 'wpbga_backend_js', $this->plugins_url( 'assets/js/google-author-backend.js', $this->_plugin_file ), array( 'jquery' ), $this->_plugin_version );
		wp_enqueue_script( 'wpbga_backend_js' );

		wp_register_style( 'wpbga_backend_style', $this->plugins_url( 'assets/css/google-author-backend.css' ), false, $this->_plugin_version );
		wp_enqueue_style( 'wpbga_backend_style' );

		// always include the wp-pointer
		wp_enqueue_style( 'wp-pointer' );
		wp_enqueue_script( 'wp-pointer' );
	}


	/**
	 * Adds Javascript to the footer which displays the WPPointer Tooltip
	 * @since 1.2
	 * @return void
	 */
	public function wp_pointer_message() {
		global $pagenow;

		if ( ! ( 'options-general.php' == $pagenow && isset( $_REQUEST['page'] ) && str_replace( 'settings_page_', '', $this->_pagehook ) == $_REQUEST['page'] ) ) {
			return;
		}

		$options = get_option( 'gpaisrpro' );
		if ( isset( $options['help_pointer'] ) && 1 == $options['help_pointer'] ) {
			return;
		}

		$popup_content = '<h3>' . __( 'Need help?', $this->_plugin_textdomain ) . '</h3>';
		$popup_content .= '<p>' . __( 'Click here to expand the help section.', $this->_plugin_textdomain ) . '</p>';
		?>
		<script type="text/javascript">
			//<![CDATA[

			(function ($) {
				var wpbga_pointer_options, setup, button;

				wpbga_pointer_options = $.extend(wpbga_pointer_options, {
					'position': {'edge': 'top', 'align': 'left'},
					'content' : '<?php echo $popup_content; ?>',
					buttons   : function (event, t) {
						button = jQuery('<a id="pointer-close" style="margin-right:5px" class="button-secondary"><?php echo __( 'Not now!', $this->_plugin_textdomain ); ?></a>');
						button.bind('click.pointer', function () {
							t.element.pointer('close');
						});
						return button;
					},
					close     : function () {
					}
				});


				setup = function () {
					jQuery('#gpaisrpro_howto .hndle span').pointer(wpbga_pointer_options).pointer('open');

					jQuery('#pointer-close').click(function () {
						jQuery('#gpaisrpro_howto .hndle span').pointer('close');
					});

				};

				jQuery(document).ready(setup);
			})(jQuery);

		</script>

	<?php

	}

	/**
	 * This is the activation action
	 * @since 1.2
	 * @return void
	 */
	public function on_activation() {
		$options                 = get_option( 'gpaisrpro' );
		$options['help_pointer'] = 0;
		update_option( 'gpaisrpro', $options );
	}


	/**
	 * Returns the purchase code of the buyer
	 *
	 * @return string
	 * @since 1.2
	 */
	protected function get_purchase_code() {
		$options = get_option( 'gpaisrpro' );
		if ( isset( $options['purchase_code'] ) ) {
			return $options['purchase_code'];
		}
		return '';
	}


	/**
	 * Adds the publisher link to the page bottom
	 * @since 1.2
	 * @return void
	 */
	public function add_google_publishership() {
		$options              = get_option( 'gpaisrpro' );
		$google_plus_page_url = '';
		if ( isset( $options['google_plus_page_url'] ) ) {
			$google_plus_page_url = $options['google_plus_page_url'];
		}

		if ( empty( $google_plus_page_url ) ) {
			return;
		}

		// get the link text
		$google_plus_page_text = '';
		if ( isset( $options['google_plus_page_text'] ) ) {
			$google_plus_page_text = $options['google_plus_page_text'];
		}
		if ( empty( $google_plus_page_text ) ) {
			$google_plus_page_text = __( 'Our Google+ page', $this->get_textdomain() );
		}

		// get the link styles
		$google_plus_page_styles = '';
		if ( isset( $options['google_plus_page_styles'] ) ) {
			$google_plus_page_styles = $options['google_plus_page_styles'];
		}

		echo '<p style="' . esc_attr( $google_plus_page_styles ) . '"><a style="' . esc_attr( $google_plus_page_styles ) . '" href="' . esc_attr( $google_plus_page_url ) . '" rel="publisher" target="_blank">' . esc_attr( $google_plus_page_text ) . '</a></p>';
	}


	/**
	 * Adds the google link to the header
	 * @global WP_Post $post
	 * @since 2.3
	 */
	public function author_header() {

		global $post;

		if ( ! $post instanceof WP_Post ) {
			return;
		}

		// only for post types (singular items)
		if ( ! is_singular() ) {
			return;
		}

		$options = get_option( 'gpaisrpro' );

		// if nothing was set: posts and pages are standard
		if ( ! isset( $options['post_types'] ) ) {
			$options['post_types'] = array( 'post', 'page' );
		}

		// if the current post type does not match the selected ones, just do nothing
		if ( ! in_array( $post->post_type, $options['post_types'] ) ) {
			return;
		}

		// find the link
		$gplus_author_link = get_the_author_meta( 'googleplus', $post->post_author );
		if ( empty( $gplus_author_link ) ) {
			return;
		}

		echo '<link rel="author" href="' . $gplus_author_link . '" />';
	}


	/**
	 * Adds the publisher link to the header
	 * @since 2.3
	 */
	public function publisher_header() {
		$options = get_option( 'gpaisrpro' );
		if ( ! isset( $options['google_plus_page_url'] ) ) {
			return;
		}
		if ( empty( $options['google_plus_page_url'] ) ) {
			return;
		}

		echo '<link rel="publisher" href="' . $options['google_plus_page_url'] . '" />';
	}


	/**
	 * Adds a contact method to the user profile page
	 *
	 * @param array $contact_methods
	 *
	 * @return array
	 * @since 2.3
	 */
	public function user_contactmethods( $contact_methods ) {
		$contact_methods['googleplus'] = __( 'Google+', $this->get_textdomain() );
		return $contact_methods;
	}


	/**
	 * Do the upgrade (move all google+ links to the new field)
	 * @since 2.3
	 */
	public function on_upgrade() {

		/**
		 * The old version (< 2.3) has built an extra input-field on the user profile settings page
		 * The new versions generate an input field directly on the "Contact Info" section
		 * The upgrade down there will remove the old user meta and will copy it to the new one
		 */
		if ( version_compare( get_option( 'wpb_plugin_' . $this->get_plugin_name_sanitized() . '_version', 0 ), '2.3', '<' ) ) {
			$users = get_users();
			foreach ( $users as $user ) {
				$old_gplus_link = get_user_meta( $user->ID, 'gplus_link', true );

				// check if there is a old gplus link. if not, continue
				if ( empty( $old_gplus_link ) ) {
					continue;
				}

				$new_gplus_link = get_user_meta( $user->ID, 'googleplus', true );

				// check if there is already a gplus link in the users meta (maybe from Yoast SEO Plugin). If yes, continue
				if ( ! empty( $new_gplus_link ) ) {
					continue;
				}

				// alright. there is no gplus link, so add the new one ...
				update_user_meta( $user->ID, 'googleplus', $old_gplus_link );

				// ... and delete the old one
				delete_user_meta( $user->ID, 'gplus_link' );
			}
		}
	}

	/**
	 * Loads the posts for the select box on the settings page
	 * @since 2.4
	 * @global wpdb $wpdb
	 */
	public function ajax_postlist() {

		if ( ! isset( $_REQUEST['term'] ) ) {
			wp_send_json_error();
		}
		if ( ! isset( $_REQUEST['_wpbga_nonce_name'] ) ) {
			wp_send_json_error();
		}
		if ( ! wp_verify_nonce( $_REQUEST['_wpbga_nonce_name'], 'wp_ajax_wpbga_ajax_postlist' ) ) {
			wp_send_json_error();
		}

		global $wpdb;

		if ( ! $wpdb instanceof wpdb ) {
			wp_send_json_error();
		}

		$results = $wpdb->get_results( $wpdb->prepare( 'SELECT post_title, ID, post_type FROM `%1$s` WHERE post_title LIKE "%%%2$s%%" AND post_status="publish"', $wpdb->posts, $_REQUEST['term'] ) );

		$postlist = array();

		$post_types = get_post_types( array( 'public' => true, 'show_ui' => true ) );

		if ( is_array( $results ) ) {
			foreach ( $results as $post ) {
				$post_type = array_search( $post->post_type, $post_types );
				if ( false === $post_type ) {
					$post_type = $post->post_type;
				}
				$postlist[$post->ID] = $post->post_title . ' (' . $post_type . ' - ID: ' . $post->ID . ')';
			}
		}

		wp_send_json( array( 'terms' => $postlist ) );
	}

	/**
	 * Loads the users for the select box on the settings page
	 * @since 2.4
	 */
	public function ajax_userlist() {
		if ( ! isset( $_REQUEST['term'] ) ) {
			wp_send_json_error();
		}
		if ( ! isset( $_REQUEST['_wpbga_nonce_name'] ) ) {
			wp_send_json_error();
		}
		if ( ! wp_verify_nonce( $_REQUEST['_wpbga_nonce_name'], 'wp_ajax_wpbga_ajax_userlist' ) ) {
			wp_send_json_error();
		}

		$userlist = array();

		$users = get_users();
		foreach ( $users as $user ) {

			if ( false !== stripos( $user->display_name, $_REQUEST['term'] )
					OR false !== stripos( $user->user_nicename, $_REQUEST['term'] )
					OR false !== stripos( $user->user_login, $_REQUEST['term'] )
			) {
				$userlist[$user->ID] = $user->display_name . ' (' . $user->user_login . ')';
			}


		}

		wp_send_json( array( 'terms' => $userlist ) );

	}

	/**
	 * adds author headers to tag, category and other special pages
	 * @since 2.4
	 */
	public function author_header_special() {
		$options = get_option( 'gpaisrpro' );

		if ( ( is_front_page() OR is_home() ) && isset( $options['google_plus_frontpage_users'] ) ) {
			foreach ( $options['google_plus_frontpage_users'] as $frontpage_user ) {

				$gplus_author_link = get_the_author_meta( 'googleplus', $frontpage_user );
				if ( empty( $gplus_author_link ) ) {
					continue;
				}
				echo '<link rel="author" href="' . $gplus_author_link . '" />';
			}
		}

		if ( is_category() && isset( $options['google_plus_category_users'] ) ) {
			foreach ( $options['google_plus_category_users'] as $category_user ) {

				$gplus_author_link = get_the_author_meta( 'googleplus', $category_user );
				if ( empty( $gplus_author_link ) ) {
					continue;
				}
				echo '<link rel="author" href="' . $gplus_author_link . '" />';
			}
		}

		if ( is_tag() && isset( $options['google_plus_tag_users'] ) ) {
			foreach ( $options['google_plus_tag_users'] as $tag_user ) {

				$gplus_author_link = get_the_author_meta( 'googleplus', $tag_user );
				if ( empty( $gplus_author_link ) ) {
					continue;
				}
				echo '<link rel="author" href="' . $gplus_author_link . '" />';
			}
		}

		if ( is_author() ) {
			$gplus_author_link = get_the_author_meta( 'googleplus' );
			if ( ! empty( $gplus_author_link ) ) {
				echo '<link rel="author" href="' . $gplus_author_link . '" />';
			}
		}

	}
}
